@@ -26,6 +26,7 @@ def uncamel(name):
 
 class APIDefinition(object):
     url = ''
+    key_name = 'id'
     actions = []
     extra_actions = []
     required_params = []
@@ -172,15 +173,38 @@ class Project(APIDefinition):
         url = '/repository/branches/:branch/protect'
         method = _HTTP_PUT
         url_params = [
-            'branch',
+            'branch',  # GitLab API is branch name
         ]
+        @staticmethod
+        def wrapper(extra_action_fn, parent):
+            """Accept a Branch object instead of a branch name"""
+            def wrapped(self, branch):
+                name = getattr(branch, 'name', branch)  # allow passing name
+                extra_action_fn(self, name)
+                try: # If passed a branch object, update it
+                    setattr(branch, 'protected', True)
+                except AttributeError:
+                    pass
+            return wrapped
     class UnprotectBranchAction(ExtraActionDefinition):
         """gl.Project.unprotect_branch()"""
         url = '/repository/branches/:branch/unprotect'
         method = _HTTP_PUT
         url_params = [
-            'branch',
+            'branch',  # GitLab API is branch name
         ]
+        @staticmethod
+        def wrapper(extra_action_fn, parent):
+            """Accept a Branch object instead of a branch name"""
+            def wrapped(self, branch):
+                name = getattr(branch, 'name', branch)  # allow passing name
+                extra_action_fn(self, name)
+                try: # If passed a branch object, update it
+                    setattr(branch, 'protected', False)
+                except AttributeError:
+                    pass
+            return wrapped
+
     extra_actions = [
         ForkFromAction,
         DeleteForkAction,
@@ -199,6 +223,7 @@ class Project(APIDefinition):
         actions = [ _LIST ]
 
     class Hook(APIDefinition):
+        url = '/hooks/:id'
         actions = [ _LIST, _GET, _ADD, _EDIT, _DELETE ]
         required_params = [
             'url',
@@ -221,6 +246,7 @@ class Project(APIDefinition):
 
     class Branch(APIDefinition):
         url = '/repository/branches/:branch'
+        key_name = 'name'
         actions = [ _LIST, _GET ]
 
         @classmethod
@@ -233,10 +259,24 @@ class Project(APIDefinition):
             """gl.Project.Branch.protect()"""
             url = '/protect'
             method = _HTTP_PUT
+            @staticmethod
+            def wrapper(extra_action_fn, parent):
+                """Accept a Branch object instead of a branch name"""
+                def wrapped(self):
+                    extra_action_fn(self)
+                    setattr(self, 'protected', True)
+                return wrapped
         class UnprotectAction(ExtraActionDefinition):
             """gl.Project.Branch.unprotect()"""
             url = '/unprotect'
             method = _HTTP_PUT
+            @staticmethod
+            def wrapper(extra_action_fn, parent):
+                """Accept a Branch object instead of a branch name"""
+                def wrapped(self):
+                    extra_action_fn(self)
+                    setattr(self, 'protected', False)
+                return wrapped
         extra_actions = [ ProtectAction, UnprotectAction ]
 
     class MergeRequest(APIDefinition):
@@ -398,6 +438,16 @@ class GitLab(APIDefinition):
             'snippets_enabled',
             'public',
         ]
+
+        @staticmethod
+        def wrapper(extra_action_fn, parent):
+            def wrapped(*args, **kwargs):
+                """Return the created Project"""
+                import gitlab3
+                project_data = extra_action_fn(*args, **kwargs)
+                return gitlab3.Project(parent, project_data)
+            return wrapped
+
     extra_actions = [ AddProjectForUserAction ]
 
     sub_apis = [
